#!/usr/bin/env ruby

require "rxldata.rb"

class Array

  # min, max already exist

  def sum
    inject { |sum, n| sum = sum + n }
  end

  def mean
    sum/size
  end

  def var
    m = Float(mean)
	inject(Float(0)) { |sum, n| sum = sum + (Float(n)-m)**2 } / Float(size)
  end
  
end

class RxlEval < Object
  
  # RegExp's
  CELLS = /(<(\w+\d+)>)/					      # [["<cell>", "cell"], ... ]
  RANGES = /<\w+\d+>\s*-\s*<\w+\d+>/
  R_START = /(<(\w+\d+)>)\s*-/					# [["<a1>", "a1"]]
  R_STOP = /-\s*(<(\w+\d+)>)/
  ARRAYS = /\s*\[[^\]]+\]\s*/
  SEQUENCES = /((<\w+\d+>\s*,\s*)+<\w+\d+>)/   # Match is found in first element of each (sub-)array element
  FUNCTIONS = /((\w+)\s?\(([^\)]+)\))/         # [["fun(arg)", "fun", "arg"], ... ]
  
  # '<A1>+<B1>' --> ["A1","B1"]
  def RxlEval.cells(expr)
    expr.scan(CELLS).collect{ |a| a[1].upcase }
  end

  def RxlEval.expand_params(expr, params)
    e = expr.clone
	expr.scan(CELLS).each { |c|
      e.gsub!(c[0], params[c[1].upcase].to_s) if c[1] && params[c[1].upcase] 
    }
    e
  end
  
  # '<a1>-<a3>' --> '[<a1>,<a2>,<a3>]'
  # '<a1>-<c1>' --> '[<a1>,<b2>,<c3>]'
  def RxlEval.convert_ranges(expr)
    e = expr.clone
	expr.scan(RANGES).each { |r|
	  from, to = r.scan(R_START)[0][1], r.scan(R_STOP)[0][1]
	  fc, fr = RxlData.parse(from); tc, tr = RxlData.parse(to)
	  a = nil
	  if (fc == tc)
		a = (fr..tr).collect { |rr| fc+rr }
	  elsif (fr == tr)
		a = (fc..tc).collect { |c| c+fr }
	  else
		# invalid range
	  end
	  e.gsub!(r, RxlEval.print_array(a))
	}
	e
  end

  # Convert function to array operation. This assumes that argument is an printed array
  # 'sum([<a1>,<a2>,<a3>])' --> ' [<a1>,<a2>,<a3>].sum'
  # 'sum(<a1>,<a2>,<a3>)'   --> ' [<a1>,<a2>,<a3>].sum'
  # 'sum(1,2,3)'   --> ' [1,2,3].sum'
  def RxlEval.convert_functions(expr)
    e = expr.clone
    functions = expr.scan(FUNCTIONS).collect { |a| a[0] }
	functions.each { |f|
      fun = f.scan(FUNCTIONS)[0][1]
      arg = f.scan(FUNCTIONS)[0][2]
	  arg = '[' + arg + ']' unless (arg.scan(ARRAYS).size > 0)
	  e.gsub!(f, " " + arg + "." + fun)
    }
	e
  end

  # Evaluate rxl expr using params hash
  def RxlEval.evaluate(expr, params)
    e = expr.clone
	e = RxlEval.convert_ranges(e)
	e = RxlEval.convert_functions(e)
	e = RxlEval.expand_params(e, params)
    eval(e)
  end
  
  # ["a1","a2"] --> [<a1>,<a2>] 
  def RxlEval.print_array(ary)
	'[' + ary.to_a.inject("") { |s, e| s = s + '<'+e.to_s+'>'+',' }.chop + ']'
  end
  
  # See rxltest.rb
  def RxlEval.unit_test
    puts "RxlEval.unit_test ..."    
    puts e = RxlEval.convert_ranges("<a1>-<a3>, <a1>-<c1>")
    params = {"A1"=>1,"A2"=>2,"A3"=>3,"B1"=>4,"C1"=>5}
    puts e = RxlEval.expand_params(e, params)
    puts e = RxlEval.convert_functions("sin([<a1>,<a2>,<a3>])+cos(<a1>,<b1>,<c1>))")
    puts e = RxlEval.evaluate("sum(<a1>-<a3>)+ min(<a1>-<c1>)", params)
    puts e = RxlEval.evaluate("1+sum(<a1>- <a3>)+ min(<a1> -<c1>) +max(<a1>,<b1>,<c1>)", params)
    puts e = RxlEval.evaluate("<a1>+var(1.1,0.9,1.2)", params)
    puts e = RxlEval.evaluate("(<a1>+var(1.1,0.9,1.2))>1", params)
    puts e = RxlEval.evaluate("(<a1> > <a2>) ? \"A1 > A2\" : \"A1 < A2\"", params)
    puts e = RxlEval.evaluate("Math::PI*<a1>", params)
    puts RxlEval.cells("<a1>").to_s
    puts e = RxlEval.expand_params("<a1>+<b1>", { "A1" => 1 })
    puts e = RxlEval.expand_params("<a1>+<b1>", { "B1" => 2 })
  end
  
end
