#!/usr/bin/env ruby

class RxlGraph < Object

  # simple directed graph implementation to handle rxl dependency calculation

  def initialize()
	  @in = Hash.new	# elements, where values are an array of elements dependent on key
	  @out = Hash.new	# elements, where values are an array of elements that key is dependent on	
  end

  def add(from,to)
    @out[from] = [] unless @out[from] 
    @out[from] << to unless @out[from].include? to
    @in[to] = [] unless @in[to] 
    @in[to] << from unless @in[to].include? from
  end
  
  def remove(from,to)
    @out[from].delete to
    @out.delete from if @out[from].empty? 
    @in[to].delete from
    @in.delete to if @in[to].empty? 
  end
  
  def empty?
    (@in.empty? && @out.empty?)
  end

  def in_dep(elem)
    @in[elem] || []
  end

  alias dep_to in_dep
  
  def out_dep(elem)
    @out[elem] || []
  end

  alias dep_from out_dep
  
  # Return all elements of elems that doesn't have any outgoing dependence
  def no_out_dep(elems)
    elems - @out.keys
  end

  # Return all elements of elems that doesn't have any inward dependence
  def no_in_dep(elems)
    elems - @in.keys
  end
  
  def dependencies
    @out.inject(0) { |sum, a| sum + a.size }
  end

  # See rxltest.rb  
  def RxlGraph.unit_test
    puts "RxlGraph.unit_test ..."
    g = RxlGraph.new
    g.add("a1", "b1")
    puts "[a1]="+g.dep_to("b1").to_s
    g.add("a1", "b2")
    g.add("a1", "a2")
    g.add("a2", "b2")
    puts "[a1,a2]="+g.in_dep("b2").to_s
    puts "[b1,b2,a2]="+g.out_dep("a1").to_s
    g.remove("a1", "b2")
    puts "[b1,a2]="+g.out_dep("a1").to_s
    puts "[b1,b2]="+g.no_out_dep(["a1","a2","b1","b2"]).to_s
    puts "[a1]="+g.no_in_dep(["a1","a2","b1","b2"]).to_s
    puts "[]="+g.no_in_dep([]).to_s
    g.remove("a1", "b1")
    g.remove("a1", "a2")
    g.remove("a2", "b2")
    puts "empty?="+g.empty?.to_s
  end
  
end
